@@ -95,7 +95,8 @@ _pygi_scan_for_callbacks (GIFunctionInfo *function_info,
             interface_type = g_base_info_get_type (interface_info);
             if (interface_type == GI_INFO_TYPE_CALLBACK &&
                     ! (strcmp (g_base_info_get_namespace ( (GIBaseInfo*) interface_info), "GLib") == 0 &&
-                       strcmp (g_base_info_get_name ( (GIBaseInfo*) interface_info), "DestroyNotify") == 0)) {
+                       (strcmp (g_base_info_get_name ( (GIBaseInfo*) interface_info), "DestroyNotify") == 0 ||
+                       (strcmp (g_base_info_get_name ( (GIBaseInfo*) interface_info), "FreeFunc") == 0)))) {
                 if (*callback_index != G_MAXUINT8) {
                     PyErr_Format (PyExc_TypeError, "Function %s.%s has multiple callbacks, not supported",
                                   g_base_info_get_namespace ( (GIBaseInfo*) function_info),
@@ -108,11 +109,8 @@ _pygi_scan_for_callbacks (GIFunctionInfo *function_info,
             g_base_info_unref (interface_info);
         }
         destroy = g_arg_info_get_destroy (arg_info);
-        if (is_method)
-            --destroy;
+        
         closure = g_arg_info_get_closure (arg_info);
-        if (is_method)
-            --closure;
         direction = g_arg_info_get_direction (arg_info);
 
         if (destroy > 0 && destroy < n_args) {
@@ -200,7 +198,8 @@ _pygi_create_callback (GIBaseInfo  *function_info,
 
     if (!found_py_function
             || (py_function == Py_None || !PyCallable_Check (py_function))) {
-        PyErr_Format (PyExc_TypeError, "Error invoking %s.%s: Invalid callback given for argument %s",
+        PyErr_Format (PyExc_TypeError, "Error invoking %s.%s: Unexpected value "
+                      "for argument '%s'",
                       g_base_info_get_namespace ( (GIBaseInfo*) function_info),
                       g_base_info_get_name ( (GIBaseInfo*) function_info),
                       g_base_info_get_name ( (GIBaseInfo*) callback_arg));
